﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Infragistics.Win;
using System.ComponentModel;
using Infragistics.Win.Misc;
using Infragistics.Win.FormattedLinkLabel;
using System.Drawing;

namespace WinTilePanelCreationFilter
{
    class MyCreation:IUIElementCreationFilter,INotifyPropertyChanged
    {
        public delegate void HeaderClickedHandler(object sender, TileClickArgs args);
        public event HeaderClickedHandler HeaderClicked;
        

        #region IUIElementCreationFilter Members

        Dictionary<UltraTile, UltraTileControlUIElement> tileUI = new Dictionary<UltraTile, UltraTileControlUIElement>();
            public void  AfterCreateChildElements(UIElement parent)
            {
                NotifyPropertyChanged(GetProperyeFormated(parent, "AfterCreateChildElements"));
                UltraTileControlUIElement ut = parent as UltraTileControlUIElement;
                if (ut != null)
                {
                    if (ut.Control == null || !(ut.Control is UltraTile))
                        return;

                    if (tileUI.ContainsKey((UltraTile)ut.Control))
                    {
                        tileUI[(UltraTile)ut.Control] = ut;
                    }
                    else
                    {
                        tileUI.Add((UltraTile)ut.Control, ut);
                        ((UltraTile)ut.Control).MouseClick += new System.Windows.Forms.MouseEventHandler(tile_MouseClick);
                    }

                }
                
            }

            public bool  BeforeCreateChildElements(UIElement parent)
            {
                NotifyPropertyChanged(GetProperyeFormated(parent, "BeforeCreateChildElements"));
                return false;
            }

        #endregion

        #region INotifyPropertyChanged Members

            public event System.ComponentModel.PropertyChangedEventHandler  PropertyChanged;

            private void NotifyPropertyChanged(String info)
            {
                if (PropertyChanged != null)
                {
                    PropertyChanged(this, new System.ComponentModel.PropertyChangedEventArgs(info));
                }
            }

        #endregion

            private string GetProperyeFormated(UIElement el, string eventName)
            {
                return string.Format("{0,-30}: for element: {1}", eventName, el.GetType().Name);
            }

            void tile_MouseClick(object sender, System.Windows.Forms.MouseEventArgs e)
            {
                if(!tileUI.ContainsKey((UltraTile)sender))
                    return;
                UltraTileControlUIElement tileControlUI = tileUI[(UltraTile)sender];
                Point p = e.Location;// ((UltraTile)sender).PointToClient(e.Location);
                if( IsElementTileHeaderUIElement(tileControlUI.ElementFromPoint(p))) 
                {
                    HeaderClicked((UltraTile)sender,new TileClickArgs((UltraTile)sender));
                }

            }

            private bool IsElementTileHeaderUIElement(UIElement el)
            {
                if (el == null)
                    return false;
                if (el is TileHeaderUIElement)
                    return true;
                else
                    return IsElementTileHeaderUIElement(el.Parent);
            }
    }

    public class TileClickArgs:EventArgs
    {
        UltraTile ultraTileElement;

        public UltraTile UltraTileElement
        {
            get { return ultraTileElement; }
        }

        public TileClickArgs(UltraTile UltraTileElement)
        {
            ultraTileElement = UltraTileElement;
        }
    }
}
